#!/usr/bin/env python

import Numeric
import MLab
import f77


def checkargumentshape(args, names):
    testshape = args[0].shape
    passed = 1
    for i in args[1:]:
        if testshape != i.shape:
            passed = 0
            break
    if passed != 1:
        errm = "The shape of all " + str(len(args)) +\
               " arguments must be the same! \n"
        for i in range(len(args)):
            errm = errm + "Argument " + str(i) + " `" + names[i] +\
                   "' had a shape of " + str(args[i].shape) + " \n"
        raise TypeError, errm


    
class _FortranExponentNoise:
    def __init__(self, freq, ampl, phase, decay, t0, type):
        checkargumentshape((freq, ampl, phase, decay, t0),
                           ('Frequency', 'Amplitude', 'Phase', 'Decay', 't0'))
        self.starttime = t0

        tmp = Numeric.absolute(1./decay) * 50.0
        self.maxdecay = MLab.max(tmp)
        self.endtime  = t0 + tmp
        self.freq     = freq
        self.amplitude= ampl
        self.phase    = phase
        self.decay    = decay
        self.SetFortranFunction(type)

    def SetFortranFunction(self, type):
        self.expfortran = f77.ExponentNoise(type)
        
    def eval(self, time):
        volts = self.expfortran.eval(time, self.freq, self.amplitude,
                                self.phase,  self.decay, self.starttime,
                                self.endtime)
        return volts
    

def testExponentNoise():
    from RandomArray import uniform
    import Gnuplot

    len = 1000
    n   = 10
    maxvolts = 1

    g = Gnuplot.Gnuplot() 

    myfreq  = uniform(len*50,    len*100,  (n*len,))
    myampl  = uniform(-maxvolts, maxvolts, (n*len,))
    mydecay = uniform(-3.*len,   -1.*len,  (n*len,))
    myt0    = uniform(0.0,       1e-3,     (n*len,))
    tmp     = Numeric.arange(len) * 2 * Numeric.pi / len
    tmp     = Numeric.ravel(Numeric.multiply.outer(Numeric.ones(n),tmp))
    myt0    = tmp + myt0
    phase   = Numeric.zeros((len * n))
    tmp     = MLab.max(Numeric.absolute(1./mydecay)) * 5.0
    
    data = []
    step = 0  
    for type in ('direct', 'simplecb', 'cb1d','cb2d'):
    #bfor type in ('cb2d',):
            print "Constructing Objects"
            t = _FortranExponentNoise(myfreq, myampl, phase, mydecay, myt0,
                                      type)
            print "Evaluation"
            hlp = Numeric.arange(3*len) * 2 * Numeric.pi / len / 500.
            res = t.eval(hlp)
            data.append(Gnuplot.Data(hlp,res+step, with='line', title=type))
            step = step + 2
    apply(g.plot, data)
    raw_input()
    
def test():
    testExponentNoise()

if __name__ == '__main__':
    test()
