#!/usr/bin/env python
"""
Encapsulates the Fortran 77 Versions of the tests
"""
from Numeric import sin, exp, ones, multiply, sum, Float, ravel, where, less
from Numeric import transpose
from MLab import min, max
from ExponentNoise import expnta

from ExponentNoiseCallback import expntaf, findf, findf2d

    
class _ExponentNoiseWrapper:
    instance_counter = 0
    def __init__(self):
        self.instance_counter = self.instance_counter + 1
        
    def eval(volts, time, freq, amplitude,
                 phase,  decay, starttime, endtime):
        """
        Pure Virtual class, use a derived one for the
        implementation!
        """
    
class _ExponentNoiseDirectCall(_ExponentNoiseWrapper):
    """
    The whole Forumla is calculated in Fortran
    """
    def eval(self, time, freq, amplitude, phase,  decay, starttime, endtime):
        return expnta(time, freq, amplitude, phase,  decay, starttime, endtime)
    
class _ExponentNoiseCallBackSimple(_ExponentNoiseWrapper):
    """
    Callback Illustration. Use the sin from Numeric instead of the fortran one.
    """
    def _sin(self, angle):
        return  sin(angle)
    
    def eval(self, time, freq, amplitude, phase,  decay, starttime, endtime):
        volts = expntaf(time, freq, amplitude, phase, decay, starttime,
                        endtime, lambda x, y=self: y._sin(x))
        return volts
    
class _ExponentNoiseCallBack1d(_ExponentNoiseWrapper):
    """
    Handles one data set of freq, ampl, phase, decay for a long data sample
    of time.
    """
    def __call__(self, help1, freq, ampl, phase, decay):
        return  exp(help1 * decay) * ampl * sin(help1 * freq + phase)
    
    def eval(self, time, freq, amplitude, phase,  decay, starttime, endtime):
        volts = ones(time.shape, Float)
        findf(volts, time, freq, amplitude, phase,  decay, starttime, endtime,
              lambda x,y,z,a,b,c,s=self : s.__call__(x,y,z,a,b),
              1000)        
        return volts

def _exponentnoiseclb2d(help1, freq, ampl, phase, decay, n1, j,l):
    help1[:n1,:] = exp(help1[:n1,:] * decay[:n1,:]) * ampl[:n1,:] * sin(help1[:n1,:] * freq[:n1,:] + phase[:n1,:])
    return help1

    
class _ExponentNoiseCallBack2d(_ExponentNoiseWrapper):
    """
    Gathers datasets of freq, ampl, phase, decay for some time values. This
    allows to evaluate the sparse values as a matrix in python.
    """
    def __call__(self, help1, freq, ampl, phase, decay, n1):
        tmp = multiply.outer(ones(n1), help1)
        tmp1 = exp(tmp * decay[:n1,:]) * ampl[:n1,:]
        tmp2 = sin(tmp[:n1,:] * freq[:n1,:] + phase[:n1,:])
        return sum(tmp1*tmp2, 0) 
    
    def eval(self, time, freq, amplitude, phase,  decay, starttime, endtime):
        check, volts = findf2d(time, freq, amplitude, phase,  decay, starttime,
                        endtime, _exponentnoiseclb2d,
                        100, 100)
        if check < 0:
            print "Got to column :", check
            raise ValueError, "Not enough data forseen for each column"
        print "Used ", check, "number of columns!"
        return volts


_expevaltypedict = {'direct'   : _ExponentNoiseDirectCall,
                    'simplecb' : _ExponentNoiseCallBackSimple,
                    'cb1d'     : _ExponentNoiseCallBack1d,
                    'cb2d'     : _ExponentNoiseCallBack2d
                    }
def ExponentNoise(type):
    try:
        tmp = _expevaltypedict[type]
    except KeyError, des:
        errm = "The specified type " +  str(type) + "is unknown." +\
               "I only know about the following keys : " +\
               string.join(_expevaltypedict.keys(), "  ")        
        raise TypeError, errm
    return tmp()


if __name__ == '__main__':
    test()
