C------------------------------------------------------------------------------
C Author: Pierre Schnizer <Pierre.Schnizer@cern.ch>
C Date  : 22. May 2002.
C Purpose: Tutorial for f2py
C License: LGPL. You should have recieved a copy of the license together with 
C this software. No Warrenty at all. 
C------------------------------------------------------------------------------
C------------------------------------------------------------------------------
C Calculates the exponential voltage for a given time t according to the 
C formula:
C volt = exp(decay (t-t0)) ampl df((t-to) * freq * phase) for 
C all  t0 < time < tend
C------------------------------------------------------------------------------
      function expntf(time, freq, n, ampl, phase, decay, t0, tend, df)
      IMPLICIT NONE
      Integer n
      external df
      double precision expntf, time, freq(n), ampl(n), phase(n), 
     1                  decay(n),  t0(n), tend(n)


      Integer i
      double precision alpha, mydec, calct, tmp1, tmp2
      
      expntf = 0.0D0

      DO 400, i = 1,n
         if (time .gt. t0(i) .and. time .lt. tend(i)) then
            calct = time - t0(i)
            tmp1  = calct * freq(i) + phase(i)
            tmp2  = -10 
            CALL df(tmp1, tmp2)
            !tmp2  = dsin(tmp1)
            alpha = ampl(i) * tmp2
            mydec = alpha   * dexp(calct * decay(i))
            expntf = expntf + mydec

         endif
 400  CONTINUE
      END

C------------------------------------------------------------------------------
C Calculates expnt for each time(i)
C------------------------------------------------------------------------------
      subroutine expntaf(volts, m, time, freq, n, ampl, phase, 
     1                  decay, t0, tend, df)
      IMPLICIT NONE
      Integer m, n
      external df
      double precision expntf, time(m), freq(n), ampl(n), phase(n), 
     1                  decay(n),  t0(n), tend(n), volts(m)


      Integer i
C      write(*,*) "n,m = ", n, m 
      DO 500, i = 1,m
C         write(*,*) "i = ", i
         volts(i) = expntf(time(i), freq, n, ampl, phase, decay, 
     1                    t0, tend, df)
 500  CONTINUE
      END

C------------------------------------------------------------------------------
C Only Search for the intervals, where exp(decay * t) * sin (a * t + phi)
C has to be evaluated. Call Python to evaluate the formula. In this case it is
C done dealing with all as many data from the time vector as possible using one
C (freq, ampl, phase, decay) data set. The calling routine must allocate the 
C helper arrays to some size (l) (Only values <= m make sense.) This allows for
C memory tuning of the callback.
C------------------------------------------------------------------------------
      subroutine findf(volts, m, time, freq, n, ampl, phase, 
     1                  decay, t0, tend, pyfunc, help1, help2, l)
      IMPLICIT NONE
      Integer m, n, l
      external pyfunc
      double precision time(m), freq(n), ampl(n), phase(n), 
     1                 decay(n),  t0(n), tend(n), volts(m), 
     2                 help1(l), help2(l)      


      Integer i, j, k, l1, ind
C      write(*,*) "n,m = ", n, m, l 

C     Initalise volts to zero:
      DO 500, k = 1,m
         volts(k) = 0.0D0
 500  CONTINUE

      DO 700, k = 1, n
         j = 0
         DO 600, i = 1, m
C            write(*,*) "Testing :", i, time(i), t0(k), tend(k)
            if (time(i) .gt. t0(k) .and. time(i) .lt. tend(k)) then
               j = j + 1
C               write(*,*) " Adding time : ", time(i), "at index", j
               help1(j) = time(i) - t0(k)
               help2(j) = i
            endif
            if (j .ge. l) then
C              Help array is filled
C              Call python function for all relevant data
C               write(*,*) "in do calling pyf with ", l, " arguments"      
               CALL pyfunc(help1, l, freq(k), ampl(k), phase(k), 
     2                     decay(k))
               DO 550, l1 = 1, l
                  ind = help2(l1)
C                  write(*,*) "adding volts", help1(l1), "at index", ind
                  volts(ind)=volts(ind) + help1(l1)
 550           CONTINUE
               j = 0
            endif
 600     CONTINUE
C        Call the pyton function for the rest of the help array
         if (j .gt. 0) then 
C            write(*,*) "finishing calling pyf with ", j, " arguments"
            CALL pyfunc(help1, j, freq(k), ampl(k), phase(k), 
     2           decay(k))
            DO 650, l1 = 1, j
               ind = help2(l1)
C               write(*,*) "adding volts", help1(l1), "at index", ind
               volts(ind)=volts(ind) + help1(l1)
 650        CONTINUE
         endif 
 700  CONTINUE
      END
