C------------------------------------------------------------------------------
C  I  want  to evaluate  all  volts(i)  at the  same  time,  so  I need  2
C dimensional helps  to pass  the information. l  allows to tune  how much
C memory is used by the functions. J is similarly used, however, I assume,
C I will never need more than j.  The function will return j, what was the
C maximum needed from j.  If j is exceeded, findf2d will return a negative
C value. Its  absolute value  is equal  to the index  of time, which was 
C the last evaluated. j and l allow tuning the memory consumption of the call 
C back.
C------------------------------------------------------------------------------
C helper function
      subroutine setarray(a, n, l, value)
      IMPLICIT NONE
      integer n,l
      double precision a(n,l), value
      integer n1, l1
      DO n1 = 1, n
         DO l1 = 1, l
            a(n1,l1) = value
         ENDDO
      ENDDO
      end
C------------------------------------------------------------------
      function findf2d(volts, m, time, freq, n, ampl, phase, 
     1                 decay, t0, tend, pyfunc1, help1, help2, l,
     2                 freqh, amplh, phaseh, decayh, j)
      IMPLICIT NONE
      Integer m, n, l, findf2d, j
      external pyfunc1
      double precision time(m), freq(n), ampl(n), phase(n),
     1                 decay(n),  t0(n), tend(n), volts(m), 
     2                 help1(j,l), help2(l),
     3                 freqh(j,l), amplh(j,l), phaseh(j,l), decayh(j,l)

       Integer i, k, l1, ind, n1, l2, n1max, leval, h
       double precision mysum

       findf2d = 0
C       write(*,*) "n, m, l, j = ", n, m, l,j

C     TODO volts not evaluated only once?
C     Initalise volts to zero:
      DO 800, k = 1,m
         volts(k) = 0.0D0
 800  CONTINUE
C      write(*,*) "After 1. init "
C     Python is fast in evaluating matrices, but not the same number of rows 
C     will be used in each column, therefore I set the values, so that it 
C     evaluates to zero for these items
      CALL setarray(freqh,   j, l, 0.0D0)
      CALL setarray(amplh,   j, l, 0.0D0)
      CALL setarray(phaseh,  j, l, 0.0D0)
      CALL setarray(decayh,  j, l, 0.0D0)
      CALL setarray(help1,   j, l, 0.0D0)
C      write(*,*) "After total init "

C     line of the helper array
      l1 = 0
C     last evaluated. Set to 1, as 1 is the first line checked. And in case of
C     error the value must be negative! 
      leval = 1     
C     maximum columns used in the helper arrays.  n1max will tell python which
C     subparts of the matrix need to be evaluated
      n1max = 0

C     Loop over all lines and fill the helper arrays. If it is found, that 
C     more row items for the helper array  is needed as available, stop the
C     calculation. If all columns are full, call the function pyfunc1. 
C     Store the results into volts and continue the loop.
      DO 950, i = 1, m
C        Set n1 and n1max to zero.
         n1    = 0
         DO 930, k = 1, n
C            write(*,*) "Testing :", i, time(i), t0(k), tend(k)
            if (time(i) .gt. t0(k) .and. time(i) .lt. tend(k)) then
               if (n1 .eq. 0) then
C                 Prepare for the next line                  
                  l1 = l1 + 1
               endif
               n1 = n1 + 1
               if (n1 .gt. j) then
                  write(*,*) "j not large enough! j= ", j, "l =", l
C                More Space needed, than what the temp arrays provide.
                 findf2d = -1 * leval
                 return
               endif
               help1(n1,l1) = time(i) - t0(k)
               freqh(n1,l1) = freq(k) 
               amplh(n1,l1) = ampl(k)
               phaseh(n1,l1) = phase(k)
               decayh(n1,l1) = decay(k)
C              index for backwriting
               help2(l1) = i
C               write(*,*) "Index = ", i, " l1 = ", l1, " help =", 
C     1                     help2(l1), " n1 = ", n1
            endif
 930     CONTINUE
C        ENDDO k = 1, n
C        Maximal needed lines
         if (n1max .lt. n1) then
            n1max = n1
         endif

C        Bookkeeping, to find what was the maximum value needed   
         if (n1max .gt. findf2d) then
            findf2d = n1max
         endif         
         if (l1 .ge. l) then
C              Array full we have to evaluate
C               write(*,*) "in do calling pyf with ", l1, n1max, j,
C     1                    " arguments"
C               write(*,*) "freqh = ", freqh
            CALL pyfunc1(help1, l, freqh, amplh, phaseh, decayh, j, 
     1                   n1max)
C               write(*,*) "After calling pyf with ", l1, n1max, j,
C     1                    " arguments"
C           copy the result
            DO 940, l2 = 1, l1
               ind = help2(l2)
C              write(*,*) "adding volts at index", ind, l1
C              TODO volts not only adressed once?
               mysum = 0.0D0
               DO 935, h=1, n1max
C                  write(*,*) "1 help1(",h,",",l2,") = ", help1(h,l2)
                  mysum = mysum + help1(h,l2)
 935           CONTINUE
               volts(ind)=volts(ind) + mysum
 940        CONTINUE
C           Prepare for the next fill
            l1 = 0
            n1max = 0
C           Save the last evaluated line
            leval = i
            CALL setarray(freqh,  j, l, 0.0D0)
            CALL setarray(amplh,  j, l, 0.0D0)
            CALL setarray(phaseh, j, l, 0.0D0)
            CALL setarray(decayh, j, l, 0.0D0)
            CALL setarray(help1,  j, l, 0.0D0)
         endif
 950  CONTINUE
C     ENDDO i = 1, m
C     Call the pyton function for the rest of the help array
      if (l1 .gt. 0) then 
C         write(*,*) "finishing calling pyf with ", l1, n1max, j,
C     1    " arguments"
         CALL pyfunc1(help1, l1, freqh, amplh, phaseh, decayh, j, 
     1    n1max)
         DO 980, l2 = 1, l1
            ind = help2(l2)
C           write(*,*) "adding volts", help1(l2), "at index", ind
C           TODO volts not evaluated only once?
            mysum = 0.0D0
            DO 970, h=1, n1max
               mysum = mysum + help1(h,l2)
 970        CONTINUE
            volts(ind) = volts(ind) + mysum
 980     CONTINUE
      endif
      end
C------------------------------------------------------------------------------
C End of File
C------------------------------------------------------------------------------
