#!/usr/bin/env python
#------------------------------------------------------------------------------
# Author:   Pierre Schnizer <Pierre.Schnizer@cern.ch>
# Date:     05. 11. 2002
# Purpose:  To illustrate the usage and capability of f2py
# Version:  Not even alpha. This is illustration code, so do not use it in
#           production environement
# License:  LGPL
#------------------------------------------------------------------------------
import Numeric

import ClassIntegrate
Integrate = ClassIntegrate.Integrate

class IntegrateExpert(Integrate):
    """
    This class uses the same internal alghorithm as the class integrate,
    but it offers many methods for fine tuning.
    
    """
    def _UseOptionalInput(self, precision):
        """
        Internal Method.      
        """
        # lsodar uses a variable to find out if it should investigate the
        # additional input paramters. These are honoured here.
        self.iopt = 1
        # Make _lsodar notice the change ... state 2 means, that init was
        # allready done.
        if self.state == 2:
            self.state = 3
        
    def SetFirstStepSize(self, size):
        """
        The step size to be attempted on the first step.
        the default value is determined by the solver.

        This method can only be called before the first
        calculation!
        """
        if self.state != 1:
            raise TypeError, """This method can only be called before
            the problem is evaluated!"""
        self.rwork[5 - 1] = size
        self._UseOptionalInput()
        
    def SetMaximumStepSize(self, size):
        """
        the maximum absolute step size allowed.
        the default value is infinite.
        """
        self.rwork[6 - 1] = size
        self._UseOptionalInput()
        
    def SetMinimumStepSize(self, size):
        """
        the minimum absolute step size allowed.
        the default value is 0.  (this lower bound is not
        enforced on the final step before reaching tcrit
        when task = 4 or 5.)
        """
        self.rwork[7 - 1] = size
        self._UseOptionalInput()

    def PrintAtMethodSwitch(self, bool):
        """
         flag to generate extra printing at method switches.
         ixpr = 0 means no extra printing (the default).
         ixpr = 1 means print data on each switch.
         t, h, and nst will be printed on the same logical
         unit as used for error messages.

         Allowed Input 0 and 1

         Warning : This output will go to the device/file with logical number
         6 and thus depends which FORTRAN compiler translated the _lsodar
         extension.
        """
        assert(bool  == 0 or bool == 1)
        self.iwork[5 - 1] = bool
        self._UseOptionalInput()
        
    def SetMaximumNumberOfSteps(self, step):
        """
        Maximum number of (internally defined) steps
        allowed during one call to the solver.
        the default value is 500.
        """
        assert(step > 0)
        self.iwork[6 - 1] = step
        self._UseOptionalInput()
        
    def SetMaximumNumberOfMessages(self, step):
        """
         Maximum number of messages printed (per problem)
         warning that t + h = t on a step (h = step size).
         this must be positive to result in a non-default
         value.  the default value is 10.
        """
        #assert(step > 0)
        self.iwork[7 - 1] = step
        self._UseOptionalInput()
        
    def SetMaximumOrderForNonStiffSolver(self, order):
        """
        The maximum order to be allowed for the nonstiff
        (adams) method.  the default value is 12.
        Higher values are not accepted.
        This value is held constant during the problem.
        """
        assert(order <= 12  or order >= 0)
        self.iwork[8 - 1] = order
        self._UseOptionalInput()
        
    def SetMaximumOrderForStiffSolver(self, order):
        """
        the maximum order to be allowed for the stiff
        (bdf) method.  the default value is 5.
        Higher values are not accepted.
        This value is held constant during the problem.
        """
        assert(order <= 5  or order >= 0)
        self.iwork[9 - 1] = order
        self._UseOptionalInput()
        
    def GetLastStepSize(self):
        """
        the step size in t last used (successfully)
        """
        return self.rwork[11 - 1]

    def GetNextStepSize(self):
        """
        the step size to be attempted on the next step
        """
        return self.rwork[12 - 1]

    def GetCurrentValueOfIndependentVariable(self):
        """
         the current value of the independent variable
         which the solver has actually reached, i.e. the
         current internal mesh point in t.  on output, tcur
         will always be at least as far as the argument
         t, but may be farther (if interpolation was done).
        """
        return self.rwork[13 - 1]

    def GetToleranceFactor(self):
        """
         a tolerance scale factor, greater than 1.0,
         computed when a request for too much accuracy was
         detected (state = -3 if detected at the start of
         the problem, state = -2 otherwise).  if the relative
         and the absolute accuracy are uniformly
         scaled up by a factor of tolsf for the next call,
         then the solver is deemed likely to succeed.
         (the user may also ignore tolsf and alter the
         tolerance parameters in any other way appropriate.)
        """        
        return self.rwork[14 - 1]

    def GetValueAtLastMethodSwitch(self):
        """
        the value of t at the time of the last method
        switch, if any
        """
        return self.rwork[15 - 1]

    def GetNumberOfBorderEvaluations(self):
        return self.iwork[10 - 1]

    def GetNumberOfSteps(self):
        """
        the number of steps taken for the problem so far.
        """
        return self.iwork[11 - 1]

    def GetNumberOfIntegrandEvaluations(self):
        """
        the number of integrand evaluations for the problem so far.        
        """
        return self.iwork[12 - 1]
    def GetNumberOfJacobiEvaluations(self):
        """
        the number of jacobian evaluations (and of matrix
        lu decompositions) for the problem so far.
        """
        return self.iwork[13 - 1]

    def GetMethodOrder(self):
        """
        the method order last used (successfully)
        """
        return self.iwork[14 - 1]

    def GetNextMethodOrder(self):
        """
        the method order attempted on the next step
        """
        return self.iwork[15 - 1]

    def GetIndexOfLargestErrorMagnitude(self):
        """
        the index of the component of largest magnitude in
        the weighted local error vector ( e(i)/ewt(i) ),
        on an error return with state = -4 or -5.        
        """
        return self.iwork[16 - 1]

    def GetMethodOfLastStep(self):
        """
        the method indicator for the last successful step..
        1 means adams (nonstiff), 2 means bdf (stiff).
        """
        tmp = self.iwork[19 -1]
        if tmp == 1:
            string = 'non stiff'
        else:
            string = 'stiff'
        return tmp, string    

    def GetMethodOfNextStep(self):
        """
        the method indicator for the next step..
        1 means adams (nonstiff), 2 means bdf (stiff).
        """
        tmp = self.iwork[19 -1]
        if tmp == 1:
            string = 'non stiff'
        else:
            string = 'stiff'
        return tmp, string    

#--------------------- End Class Integrate Expert -----------------------------
