#!/usr/bin/env python
#------------------------------------------------------------------------------
# Author:   Pierre Schnizer <Pierre.Schnizer@cern.ch>
# Date:     05. 11. 2002
# Purpose:  To illustrate the usage and capability of f2py
# Version:  Not even alpha. This is illustration code, so do not use it in
#           production environement
#------------------------------------------------------------------------------
import time
import copy
import sys

import Numeric
import Gnuplot

import lsodar
Integrate = lsodar.Integrate
IntegrateExpert = lsodar.IntegrateExpert

gravity = 9.81

def BulletMovement(t, vec, weight, resistance):
    """
    Use the equation descibing a bullet

    x  ... vec[0]
    vx ... vec[1]
    y  ... vec[2]
    vy ... vec[3]

    """
    #sys.stderr.write("BM1 " + str(t) + "\n")
    vec = copy.copy(vec)

    x  = vec[0]
    vx = vec[1]
    y  = vec[2]
    vy = vec[3]

    tmp = Numeric.ones(4, Numeric.Float)
    tmp[0] =  vx
    tmp[1] =  resistance * vx
    tmp[2] =  vy
    tmp[3] =  - weight * gravity + resistance * vy 
    
    return tmp


def Border(t, vec, bullet, t2, result, index = [1,],
           last=Numeric.zeros((5,5), Numeric.Float), lastindex = [0,]):
    """
    bullet describes the movement of the second bullet
    """

    t = copy.copy(t)
    vec = copy.copy(vec)

    
    # Calculate the postion for all t2's which are not reached yet
    tindex = index[0]
    if t > t2[tindex] and t <= t2[-1]:
        indices = Numeric.less_equal(t2[tindex:], t)
        ttmp = Numeric.compress(indices, t2[tindex:])
        index[0] = int(index[0]) + len(ttmp)
        rt = list(bullet(ttmp)[0])
        rt = Numeric.transpose(Numeric.array(rt))
        map(result.append, rt)

    #Get the position of the second bullet
    r2 = bullet.GetCurrentValueOfIndependentVariable()
    r1 = r2 - bullet.GetLastStepSize()
    
    if t<r1:
        # T out of scope we have to recalculate
        test = t - last[0, :]        
        i = Numeric.argmin(Numeric.clip(test, 0.0, float('Inf')))
        ttmp =  last[0, i]
        tvec =  last[1:, i]  
        bullet.Reinit(tvec,  ttmp)
        tmp = bullet(t)
    else:
        tmp = bullet(t)
        last[0, lastindex[0]] = t                 
        
        last[1:, lastindex[0]] = tmp[0]
        lastindex[0] = (lastindex[0] + 1) % 5

    
    assert(tmp[1] == None)
    assert(tmp[2] == None)
    pos = copy.copy(tmp[0])
    tmp = pos[::2] - vec[::2]
    tmp1 = Numeric.sum(tmp ** 2) + tmp[0]
    return tmp1

def run():
    t = Numeric.arange(1000) / 1000.0 * 2
    result2 = []
    bullet2 = IntegrateExpert(BulletMovement, Numeric.array((25., -10, 0, 10)),
                              0, args=(1.0, .1))
    bullet1 = IntegrateExpert(BulletMovement, Numeric.array((0., 10, 0.0, 10)),
                              0, args=(1.0, 0.1), border=Border,
                              borderargs=(bullet2, t, result2))


    tmp, jroot, t1 = bullet1(t)
    if t1 != None:
        t = t1

    result = Numeric.transpose(tmp)
    result2 = Numeric.array(result2)

    
    tmp = t[:len(result2[:,0])]
    x = result2[:,0]


    g = Gnuplot.Gnuplot()
    command = """
    set grid
    set xlabel 'x [m]'
    set ylabel 'y [m]'
    """
    g(command)


    g.plot(Gnuplot.Data(result[:,0], result[:,2], title='b1', with='line'),
           Gnuplot.Data(result2[:,0], result2[:,2], title='b2', with='line'),
           )
    print result.shape
    print result2.shape
    
    raw_input()
    

if __name__ == '__main__':
    run()
